/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* MPIA, NEVEC - MIDI project
*
* "@(#) $Id: flag.c,v 1.3 2003/07/29 12:52:22 jaffe Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jaffe      2003-07-24 written
*
*/
#define _POSIX_SOURCE_1
#include "vltPort.h"

/*----------------------------------------------------------------------- */
/* Flag table FITS binary tables routines                       */
/*----------------------------------------------------------------------- */
#include "flag.h"

/* internal prototypes */
/* get data from table_def specific to flag_table table */
void get_flag_table_info (flag_table_def *table);

/* file global variables for table type*/
static char *tabletype = "FLAG";


/* flag_table_row functions */
flag_table_row* 
make_flag_table_row(flag_table_def* table)
/*----------------------------------------------------------------------- */
/*   Make an flag_table_row structure                           */
/*   Inputs:                                                              */
/*      table     pointer to flag_table table structure         */
/*   Returns:                                                             */
/*      An flag_table_row structure pointer ; NULL => failure   */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */
{
  int i;
  flag_table_row *out;

  /* create basic structure */
  out = (flag_table_row*)malloc(sizeof(flag_table_row));
  if (out==NULL) return out; /* allocation failure? */
  
  /* zero memory */
  memset (out, 0, sizeof(flag_table_row));

  /* Make generic row structure */
  out->raw = make_table_row_from_def(table->def);

  return out; /* done */
} /* end  make_flag_table_row */

void kill_flag_table_row (flag_table_row *row)
/*----------------------------------------------------------------------- */
/*   Delete an flag_table_row structure                         */
/*   Inputs:                                                              */
/*      row    pointer to the flag_table_row structure.         */
/*----------------------------------------------------------------------- */
{
  if (row==NULL) return;         /* anybody home */
  kill_table_row(row->raw);      /* generic row structure */
  free (row);                              /* basic structure */
} /* end  kill_flag_table_row */


/* flag_table functions */

flag_table_def* 
make_flag_table_def(fitsfile *fptr, char *date_obs, int noChan, int *ierr)
/*----------------------------------------------------------------------- */
/*   Make a new flag_table in an open FITS file                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      date_obs  Date (yyyy-mm-dd) observations began                    */
/*      noChan    number of spectral channels in observatins              */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      A flag_table_def pointer; NULL indicates failure        */
/*----------------------------------------------------------------------- */
{
  flag_table_def *out = NULL;
  int  version = 1; /* table version number */
  int  ikey, icol, i;

  if (*ierr>0) return NULL;    /* existing error condition? */

  if (fptr==NULL)     return NULL; /* sanity check */

  /* create output object */
  out = (flag_table_def*)malloc(sizeof(flag_table_def));
  if (out==NULL)     {  /* check for allocation failure */
    *ierr = 10000;
    fprintf(stderr,"ERROR creating %s table_def \n", tabletype);
    return out;
  }
  memset (out, 0, sizeof(flag_table_def)); /* zero fill */

  /* create table_def */
  out->def = make_table_def(FLAG_TABLE, 0, 0);
  if (out->def==NULL) { /* check for failure */
    kill_flag_table_def(out); /* clean up */
    *ierr = 10000;
    fprintf(stderr,"ERROR creating FLAG_TABLE for %s table_def \n", 
	    tabletype);
    return NULL;
  }

  /* add keyword values */
  ikey = table_find_key(out->def, "REVISION");
  if ((ikey>0) && (*ierr==0)) 
    table_put_keyword (out->def, ikey, (void*)&version, ierr);
  ikey = table_find_key(out->def, "DATE-OBS");
  if ((ikey>0) && (*ierr==0)) 
    table_put_keyword (out->def, ikey, (void*)date_obs, ierr);
  ikey = table_find_key(out->def, "NO_CHAN");
  if ((ikey>0) && (*ierr==0)) 
    table_put_keyword (out->def, ikey, (void*)&noChan, ierr);
  /* get info from table descriptor */
  get_flag_table_info (out);

  /* open source table */
  make_table_from_def (fptr, out->def, ierr);

  return out; /* done */
} /* end make_flag_table */

flag_table_def* 
open_flag_table(fitsfile *fptr, int *ierr)
/*----------------------------------------------------------------------- */
/*   Open an existing flag_table in the active FITS file        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An flag_table_def pointer; NULL indicates failure       */
/*----------------------------------------------------------------------- */
{
  flag_table_def *out = NULL;

  if (*ierr>0) return NULL;    /* existing error condition? */
  if (fptr==NULL) return NULL; /* sanity check */

  /* create output object */
  out = (flag_table_def*)malloc(sizeof(flag_table_def));
  if (out==NULL)     {  /* check for allocation failure */
    *ierr = 10000;
    fprintf(stderr,"ERROR creating %s table_def \n", tabletype);
    return out;
  }
  memset (out, 0, sizeof(flag_table_def)); /* zero fill */

  /* position FITS file at  table */
  if (fits_movnam_hdu(fptr, BINARY_TBL, tabletype, 0, ierr)) {
    fprintf(stderr,"ERROR %d finding %s table \n", *ierr, tabletype);
    kill_flag_table_def(out); /* clean up */
    *ierr = 10000;
    return NULL;
  }

  /* get table descriptor */
  out->def = make_table_def_from_table (fptr);
  if (out->def==NULL) { /* error check */
    fprintf(stderr,"ERROR making table def for %s \n", tabletype);
    kill_flag_table_def(out); /* clean up */
    *ierr = 10000;
    return NULL;
  }

  /* get info from table descriptor */
  get_flag_table_info (out);

  return out; /* done */
} /* end open_flag_table */

int read_flag_table(fitsfile *fptr, flag_table_def* table, 
			    flag_table_row *row, long row_number,
			    int *ierr)
/*----------------------------------------------------------------------- */
/*   Read the next row from an flag_table                       */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */
{
  table_row *trow;

  if (*ierr>0) return 1;     /* existing error condition? */
  if (fptr==NULL) return 1;  /* sanity check */
  if (table==NULL) return 1; /* sanity check */
  if (row==NULL) return 1;   /* sanity check */

  /* set row number */
  if (row_number>0) { /* explicit row number */
    row->raw->trow = row_number;
    table->row = row_number;
  } else { /* next row */
    row->raw->trow = ++table->row;
  }

  /* read row */
  table_read_row(fptr, table->def, row->raw, ierr);
  if (*ierr>0) { /* error check */
    fprintf(stderr,"ERROR  read_flag_table: reading table %s \n", 
	    tabletype);
    return 1;
  }

  /* copy relevant data from generic table_row */
  trow = row->raw;
  row->timeRange[0]   = trow->data[table->timeRange_col]->tdata.tdouble[0];
  row->timeRange[1]   = trow->data[table->timeRange_col]->tdata.tdouble[1];
  row->sourceId       = trow->data[table->source_col]->tdata.tshort[0];
  row->telescope[0]      = trow->data[table->telescope_col]->tdata.tshort[0];
  row->telescope[1]      = trow->data[table->telescope_col]->tdata.tshort[1];
  row->chans[0]       = trow->data[table->chans_col]->tdata.tshort[0];
  row->chans[1]       = trow->data[table->chans_col]->tdata.tshort[1];
  strncpy(row->reason, trow->data[table->reason_col]->tdata.tchar, 24);

  return 0; /* OK - done */
} /* end read_sourcetable */

int write_flag_table(fitsfile *fptr, flag_table_def* table, 
			     flag_table_row *row, long row_number, 
			     int *ierr)
/*----------------------------------------------------------------------- */
/*   Write  the next row to an flag_table                               */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */
{
  table_row *trow;

  if (*ierr>0) return 1;     /* existing error condition? */
  if (fptr==NULL) return 1;  /* sanity check */
  if (table==NULL) return 1; /* sanity check */
  if (row==NULL) return 1;   /* sanity check */

  /* set row number */
  if (row_number>0) { /* explicit row number */
    row->raw->trow = row_number;
    table->row = row_number;
  } else { /* next row */
    row->raw->trow = ++table->row;
  }

  /* copy relevant data to generic table_row */
  trow = row->raw;
  trow->data[table->timeRange_col]->tdata.tdouble[0] = row->timeRange[0];
  trow->data[table->timeRange_col]->tdata.tdouble[1] = row->timeRange[1];
  trow->data[table->telescope_col]->tdata.tshort[0]  = row->telescope[0];
  trow->data[table->telescope_col]->tdata.tshort[1]  = row->telescope[1];
  trow->data[table->source_col]->tdata.tshort[0]    = row->sourceId;
  trow->data[table->chans_col]->tdata.tshort[0]     = row->chans[0];
  trow->data[table->chans_col]->tdata.tshort[1]     = row->chans[1];
  strncpy(trow->data[table->reason_col]->tdata.tchar, row->reason, 24);
  /* write row */
  table_write_row(fptr, table->def, row->raw, ierr);
  if (*ierr>0) { /* error check */
    fprintf(stderr,"ERROR  write_flag_table: writing table %s \n", 
	    tabletype);
    return 1;
  }

  return 0; /* OK - done */
} /* end write_flag_table */

int close_flag_table(fitsfile *fptr, flag_table_def* table, 
			     int *ierr)
/*----------------------------------------------------------------------- */
/*   Close an open flag_table, associated structures are        */
/*   destroyed                                                            */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */
{
  if (*ierr>0) return 1;    /* existing error condition? */
  if (fptr==NULL) return 1; /* sanity check */

  /* do actual file close */
  fits_close_file (fptr, ierr);

  /* delete structures */
  kill_flag_table_def(table);

  return 0; /* OK - done */
} /* end close_flag_table */

void kill_flag_table_def (flag_table_def* table)
/*----------------------------------------------------------------------- */
/*   Delete an flag_table structure                             */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */
{
  if (table==NULL) return; /* sanity check */

  /* table definition structure */
  kill_table_def(table->def);

  /* free basic structure */
  free (table);
} /* end kill_flag_table_def */

/*************************private functions ******************************/

void get_flag_table_info (flag_table_def *table)
/*----------------------------------------------------------------------- */
/*   Get data from table_def specific to flag_table table       */
/*   Uses detector member which must already be attached                  */
/*   Inputs:                                                              */
/*      table     table definitions structure                             */
/*----------------------------------------------------------------------- */
{
  int ikey, ierr=0;
  if (table==NULL) return;             /* sanity check */

  /* keyword values */
  ikey = table_find_key(table->def, "DATE-OBS");
  if (ikey>0) table->date_obs = (char*)table->def->keydata[ikey-1];
  ikey = table_find_key(table->def, "NO_CHAN");
  if (ikey>0) table->noChan = (int)table->def->keydata[ikey-1];

  /* lookup column numbers (0-rel) */
  table->timeRange_col = table_find_col (table->def, "TIMERANG")-1;
  table->source_col    = table_find_col (table->def, "SOURCE_ID")-1;
  table->telescope_col = table_find_col (table->def, "TELESCOPE")-1;
  table->chans_col     = table_find_col (table->def, "CHANS")-1;
  table->reason_col    = table_find_col (table->def, "REASON")-1;

} /* end  get_flag_table_info */

/* */
